"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.sendStatusChangedCallback = exports.sendClosedCallback = exports.sendChangedCallback = exports.callbackHandling = exports.statusChangedCallbacks = exports.closedCallbacks = exports.changedCallbacks = exports.callbackUrl = void 0;
const node_fetch_1 = __importDefault(require("node-fetch"));
const config_1 = require("../config");
const data_source_1 = require("../data_source");
const types_1 = require("../generated/types");
const globals_1 = require("../globals");
const model_1 = require("../model");
const errors_1 = require("../types/errors");
const format_1 = require("./database/format");
const signaling_1 = require("./signaling");
exports.callbackUrl = config_1.config.BASE_URL +
    (config_1.config.BASE_URL.endsWith('/') ? '' : '/') +
    'callbacks/device';
exports.changedCallbacks = new Map();
exports.closedCallbacks = new Map();
exports.statusChangedCallbacks = new Map();
/**
 * This function adds the endpoint for incoming callbacks registered by the peerconnection service.
 * @param app The express application the callback endpoint should be added to.
 */
function callbackHandling(app) {
    // TODO: adapt callback handling after codegen update
    app.post('/callbacks/device', async (req, res, next) => {
        try {
            const callback = req.body;
            if (typeof callback !== 'object')
                throw new errors_1.MalformedBodyError('Body of callback is not an object', 400);
            const callbackType = getCallbackType(callback);
            switch (callbackType) {
                case 'event':
                    return res.status(await handleEventCallback(callback)).send();
                default:
                    throw new errors_1.InvalidValueError(`Callbacks of type "${req.body.callbackType}" are not supported`, 400);
            }
        }
        catch (error) {
            return next(error);
        }
    });
}
exports.callbackHandling = callbackHandling;
/**
 * This function attempts to get the type of an incoming callback.
 * @param callback The incoming callback of which to get the type.
 * @throws {MalformedBodyError} Thrown if the callback is malformed.
 * @returns The type of the incoming callback.
 */
function getCallbackType(callback) {
    if (typeof callback.callbackType !== 'string') {
        throw new errors_1.MalformedBodyError('Property "callbackType" needs to be of type string', 400);
    }
    if (!callback.callbackType) {
        throw new errors_1.MalformedBodyError('Callbacks require property "callbackType"', 400);
    }
    return callback.callbackType;
}
/**
 * This function handles an incoming event callback.
 * @param callback The incoming event callback to be handled.
 * @throws {MalformedBodyError} Thrown if the callback is malformed.
 * @throws {InvalidValueError} Thrown if the type of the event callback is unknown.
 * @returns The status code of the callback response.
 */
async function handleEventCallback(callback) {
    if (!callback.eventType) {
        throw new errors_1.MalformedBodyError('Callbacks of type "event" require property "eventType"', 400);
    }
    if (typeof callback.eventType !== 'string') {
        throw new errors_1.MalformedBodyError('Property "callbackType" needs to be of type "string"', 400);
    }
    switch (callback.eventType) {
        case 'device-changed':
            return await handleDeviceChangedEventCallback(callback);
        default:
            throw new errors_1.InvalidValueError(`Event-callbacks of type "${callback.eventType}" are not supported`, 400);
    }
}
/**
 * This function handles an incoming "device-changed" event callback.
 * @param callback The incoming "device-changed" callback to be handled.
 * @throws {MalformedBodyError} Thrown if the callback is malformed.
 * @throws {InvalidValueError} Thrown if the device is not of type "device".
 * @returns The status code for the response to the incoming callback.
 */
async function handleDeviceChangedEventCallback(callback) {
    if (!callback.device) {
        throw new errors_1.MalformedBodyError('Event-callbacks of type "device-changed" require property "device"', 400);
    }
    const device = callback.device;
    if (!(0, types_1.isConcreteDevice)(device) &&
        !(0, types_1.isDeviceGroup)(device) &&
        !(0, types_1.isInstantiableBrowserDevice)(device) &&
        !(0, types_1.isInstantiableCloudDevice)(device)) {
        throw new errors_1.MalformedBodyError('Property "device" is not a valid device', 400);
    }
    if (!device.url) {
        throw new errors_1.MalformedBodyError('Property "device" is missing url', 400);
    }
    if (!(0, types_1.isConcreteDevice)(device)) {
        throw new errors_1.InvalidValueError(`Device needs to be of type "device" but is of type "${device.type}"`, 400 // NOTE: error code
        );
    }
    const pendingConnectionsA = await data_source_1.AppDataSource.getRepository(model_1.PeerconnectionModel).find({
        where: {
            deviceA: {
                url: device.url
            }
        },
        relations: {
            deviceA: true,
            deviceB: true
        }
    });
    const pendingConnectionsB = await data_source_1.AppDataSource.getRepository(model_1.PeerconnectionModel).find({
        where: {
            deviceB: {
                url: device.url
            }
        },
        relations: {
            deviceA: true,
            deviceB: true
        }
    });
    const pendingConnections = [...pendingConnectionsA, ...pendingConnectionsB];
    if (pendingConnections.length === 0) {
        return 410;
    }
    for (const pendingConnection of pendingConnections) {
        const deviceA = await globals_1.apiClient.getDevice(pendingConnection.deviceA.url);
        const deviceB = await globals_1.apiClient.getDevice(pendingConnection.deviceB.url);
        if (deviceA.connected && deviceB.connected) {
            clearTimeout(globals_1.timeoutMap.get(pendingConnection.uuid));
            await (0, signaling_1.startSignaling)(pendingConnection);
            globals_1.timeoutMap.delete(pendingConnection.uuid);
        }
    }
    return pendingConnections.length === 0 ? 410 : 200;
}
/**
 * This function sends a "device-changed" callback.
 * @param device The device for which to send the callback.
 */
async function sendChangedCallback(device) {
    console.log(`Sending changedCallback for device ${device.uuid}`);
    const urls = exports.changedCallbacks.get(device.uuid) ?? [];
    for (const url of urls) {
        console.log(`Sending changedCallback for device ${device.uuid} to url ${url}`);
        const res = await (0, node_fetch_1.default)(url, {
            method: 'POST',
            body: JSON.stringify({
                callbackType: 'event',
                eventType: 'device-changed',
                device: await (0, format_1.formatDevice)(device),
            }),
            headers: [
                ["Content-Type", "application/json"]
            ]
        });
        if (res.status === 410) {
            const changedCallbackURLs = exports.changedCallbacks.get(device.uuid) ?? [];
            exports.changedCallbacks.set(device.uuid, changedCallbackURLs.filter((cb_url) => cb_url != url));
        }
    }
}
exports.sendChangedCallback = sendChangedCallback;
/**
 * This function sends a "peerconnection-closed" callback.
 * @param peerconnection The peerconnection for which to send the callback.
 */
async function sendClosedCallback(peerconnection) {
    const urls = exports.closedCallbacks.get(peerconnection.uuid) ?? [];
    for (const url of urls) {
        const res = await (0, node_fetch_1.default)(url, {
            method: 'post',
            body: JSON.stringify({
                callbackType: 'event',
                eventType: 'peerconnnection-closed',
                peerconnection: (0, format_1.formatPeerconnection)(peerconnection),
            }),
            headers: [
                ["Content-Type", "application/json"]
            ]
        });
        if (res.status === 410) {
            const closedCallbackURLs = exports.closedCallbacks.get(peerconnection.uuid) ?? [];
            exports.closedCallbacks.set(peerconnection.uuid, closedCallbackURLs.filter((cb_url) => cb_url != url));
        }
    }
}
exports.sendClosedCallback = sendClosedCallback;
/**
 * This function sends a "peerconnection-status-changed" callback.
 * @param peerconnection The peerconnection for which to send the callback.
 */
async function sendStatusChangedCallback(peerconnection) {
    console.log(`Sending statusChangedCallback for peerconnection ${peerconnection.uuid}`);
    const urls = exports.statusChangedCallbacks.get(peerconnection.uuid) ?? [];
    for (const url of urls) {
        console.log(`Sending statusChangedCallback for peerconnection ${peerconnection.uuid} to url ${url}`);
        const res = await (0, node_fetch_1.default)(url, {
            method: 'post',
            body: JSON.stringify({
                callbackType: 'event',
                eventType: 'peerconnection-status-changed',
                peerconnection: (0, format_1.formatPeerconnection)(peerconnection),
            }),
            headers: [
                ["Content-Type", "application/json"]
            ]
        });
        if (res.status === 410) {
            const statusCallbackURLs = exports.statusChangedCallbacks.get(peerconnection.uuid) ?? [];
            exports.statusChangedCallbacks.set(peerconnection.uuid, statusCallbackURLs.filter((cb_url) => cb_url != url));
        }
    }
}
exports.sendStatusChangedCallback = sendStatusChangedCallback;
//# sourceMappingURL=callbacks.js.map