"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createDeviceModelFromDevice = exports.createDeviceModel = void 0;
const data_source_1 = require("../../data_source");
const types_1 = require("../../generated/types");
const model_1 = require("../../model");
const errors_1 = require("../../types/errors");
const write_1 = require("./write");
/**
 * This function attempts to create a device model with a given type.
 * @param type The type of device model to create.
 * @throws {InvalidValueError} Thrown if the given type does not match any known device type.
 * @returns The created device model.
 */
function createDeviceModel(type) {
    switch (type) {
        case 'device':
            return data_source_1.AppDataSource.getRepository(model_1.ConcreteDeviceModel).create();
        case 'group':
            return data_source_1.AppDataSource.getRepository(model_1.DeviceGroupModel).create();
        case 'edge instantiable':
            return data_source_1.AppDataSource.getRepository(model_1.InstantiableBrowserDeviceModel).create();
        case 'cloud instantiable':
            return data_source_1.AppDataSource.getRepository(model_1.InstantiableCloudDeviceModel).create();
        default:
            throw new errors_1.InvalidValueError('The given type does not match any known device type', 500);
    }
}
exports.createDeviceModel = createDeviceModel;
/**
 * This function attempts to create a device model with the data provided.
 * @param data The provided data in the form of a device.
 * @throws {InvalidValueError} Thrown if the type of the device does not match any known device type
 * @returns The created device model.
 */
function createDeviceModelFromDevice(data) {
    if ((0, types_1.isConcreteDevice)(data)) {
        const deviceModel = createDeviceModel('device');
        (0, write_1.writeConcreteDevice)(deviceModel, data);
        return deviceModel;
    }
    else if ((0, types_1.isDeviceGroup)(data)) {
        const deviceModel = createDeviceModel('group');
        (0, write_1.writeDeviceGroup)(deviceModel, data);
        return deviceModel;
    }
    else if ((0, types_1.isInstantiableBrowserDevice)(data)) {
        const deviceModel = createDeviceModel('edge instantiable');
        (0, write_1.writeInstantiableBrowserDevice)(deviceModel, data);
        return deviceModel;
    }
    else if ((0, types_1.isInstantiableCloudDevice)(data)) {
        const deviceModel = createDeviceModel('cloud instantiable');
        (0, write_1.writeInstantiableCloudDevice)(deviceModel, data);
        return deviceModel;
    }
    else {
        throw new errors_1.InvalidValueError('The device does not match any known device type', 500);
    }
}
exports.createDeviceModelFromDevice = createDeviceModelFromDevice;
//# sourceMappingURL=create.js.map