"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.formatPeerconnection = exports.formatPeerconnectionOverview = exports.formatConfiguredDeviceReference = exports.formatDeviceReference = exports.formatServiceConfig = exports.formatDeviceOverview = exports.formatTimeSlot = exports.formatDevice = void 0;
const model_1 = require("../../model");
const errors_1 = require("../../types/errors");
const utils_1 = require("../utils");
// Devices
/**
 * This function attempts to format a given device model to its corresponding device type.
 * @param deviceModel The device model to be formatted.
 * @throws {InvalidValueError} Thrown if the given device model does not match any known device type.
 * @returns The formatted device model.
 */
async function formatDevice(deviceModel) {
    if ((0, model_1.isConcreteDeviceModel)(deviceModel)) {
        return formatConcreteDevice(deviceModel);
    }
    else if ((0, model_1.isDeviceGroupModel)(deviceModel)) {
        return (await formatDeviceGroup(deviceModel));
    }
    else if ((0, model_1.isInstantiableBrowserDeviceModel)(deviceModel)) {
        return formatInstantiableBrowserDevice(deviceModel);
    }
    else if ((0, model_1.isInstantiableCloudDeviceModel)(deviceModel)) {
        return formatInstantiableCloudDevice(deviceModel);
    }
    else {
        throw new errors_1.InvalidValueError('The device model to be formatted does not match any known device type', 500);
    }
}
exports.formatDevice = formatDevice;
/**
 * This function formats a {@link TimeSlotModel} to a {@link TimeSlot}.
 * @param timeSlot The {@link TimeSlotModel} to be formatted.
 * @returns The resulting {@link TimeSlot}.
 */
function formatTimeSlot(timeSlot) {
    return {
        start: new Date(timeSlot.start).toISOString(),
        end: new Date(timeSlot.end).toISOString(),
    };
}
exports.formatTimeSlot = formatTimeSlot;
/**
 * This function formats a {@link DeviceOverviewModel} to a {@link DeviceOverview}.
 * @param device The {@link DeviceOverviewModel} to be formatted.
 * @returns The resulting {@link DeviceOverview}.
 */
function formatDeviceOverview(device) {
    return {
        url: (0, utils_1.deviceUrlFromId)(device.uuid),
        name: device.name,
        description: device.description,
        type: device.type,
        owner: device.owner,
    };
}
exports.formatDeviceOverview = formatDeviceOverview;
/**
 * This function formats a {@link ConcreteDeviceModel} to a {@link ConcreteDevice}.
 * @param device The {@link ConcreteDeviceModel} to be formatted.
 * @returns The resulting {@link ConcreteDevice}.
 */
function formatConcreteDevice(device) {
    return {
        url: (0, utils_1.deviceUrlFromId)(device.uuid),
        name: device.name,
        description: device.description,
        type: device.type,
        owner: device.owner,
        announcedAvailability: device.announcedAvailability
            ? device.announcedAvailability.map(formatTimeSlot)
            : undefined,
        connected: device.connected !== undefined ? device.connected : undefined,
        experiment: device.experiment ? device.experiment : undefined,
        services: device.services
            ? device.services.map((sd) => JSON.parse(sd.description))
            : undefined,
    };
}
/**
 * This function formats a {@link DeviceGroupModel} to a {@link DeviceGroup}.
 * @param deviceGroup The {@link DeviceGroupModel} to be formatted.
 * @param flat_group If true then the formatted {@link DeviceGroup} will contain no further device groups.
 * @returns The resulting {@link DeviceGroup}.
 */
async function formatDeviceGroup(deviceGroup, flat_group = false) {
    const devices = [];
    if (deviceGroup.devices) {
        for (const d of deviceGroup.devices) {
            const resolvedDevice = await (0, utils_1.resolveDeviceReference)(d, flat_group);
            if (resolvedDevice) {
                if (resolvedDevice.type != 'group')
                    devices.push(resolvedDevice);
                if (resolvedDevice.type == 'group' && !flat_group)
                    devices.push(resolvedDevice);
                if (resolvedDevice.type == 'group' && flat_group)
                    devices.push(...(0, utils_1.flattenDeviceGroup)(resolvedDevice));
            }
        }
    }
    return {
        url: (0, utils_1.deviceUrlFromId)(deviceGroup.uuid),
        name: deviceGroup.name,
        description: deviceGroup.description,
        type: deviceGroup.type,
        owner: deviceGroup.owner,
        devices: devices.filter((v, i, s) => s.findIndex((d) => d.url == v.url) == i),
    };
}
/**
 * This function formats a {@link InstantiableBrowserDeviceModel} to a {@link InstantiableBrowserDevice}.
 * @param instantiableBrowserDevice The {@link InstantiableBrowserDeviceModel} to be formatted.
 * @returns The resulting {@link InstantiableBrowserDevice}.
 */
function formatInstantiableBrowserDevice(instantiableBrowserDevice) {
    return {
        url: (0, utils_1.deviceUrlFromId)(instantiableBrowserDevice.uuid),
        name: instantiableBrowserDevice.name,
        description: instantiableBrowserDevice.description,
        type: instantiableBrowserDevice.type,
        owner: instantiableBrowserDevice.owner,
        code_url: instantiableBrowserDevice.codeUrl,
    };
}
/**
 * This function formats a {@link InstantiableCloudDeviceModel} to a {@link instantiableCloudDevice}.
 * @param instantiableCloudDevice The {@link InstantiableCloudDeviceModel} to be formatted.
 * @returns The resulting {@link instantiableCloudDevice}.
 */
function formatInstantiableCloudDevice(instantiableCloudDevice) {
    return {
        url: (0, utils_1.deviceUrlFromId)(instantiableCloudDevice.uuid),
        name: instantiableCloudDevice.name,
        description: instantiableCloudDevice.description,
        type: instantiableCloudDevice.type,
        owner: instantiableCloudDevice.owner,
        instantiate_url: instantiableCloudDevice.instantiateUrl,
    };
}
// Peerconnections
/**
 * This function formats a {@link ServiceConfigModel} to a {@link ServiceConfig}.
 * @param serviceConfig The {@link ServiceConfigModel} to be formatted.
 * @returns The resulting {@link ServiceConfig}.
 */
function formatServiceConfig(serviceConfig) {
    return {
        ...JSON.parse(serviceConfig.config ?? '{}'),
        serviceType: serviceConfig.serviceType,
        serviceId: serviceConfig.serviceId,
        remoteServiceId: serviceConfig.remoteServiceId,
    };
}
exports.formatServiceConfig = formatServiceConfig;
/**
 * This function formats a {@link DeviceReferenceModel} to a {@link DeviceReference}.
 * @param device The {@link DeviceReferenceModel} to be formatted.
 * @returns The resulting {@link DeviceReference}.
 */
function formatDeviceReference(device) {
    return {
        url: device.url,
    };
}
exports.formatDeviceReference = formatDeviceReference;
/**
 * This function formats a {@link DeviceReferenceModel} to a {@link ConfiguredDeviceReference}.
 * @param device The {@link DeviceReferenceModel} to be formatted.
 * @returns The resulting {@link ConfiguredDeviceReference}.
 */
function formatConfiguredDeviceReference(device) {
    return {
        url: device.url,
        config: device.config
            ? {
                services: device.config.map(formatServiceConfig),
            }
            : undefined,
    };
}
exports.formatConfiguredDeviceReference = formatConfiguredDeviceReference;
/**
 * This function formats a {@link PeerconnectionModel} to a {@link PeerconnectionOverview}.
 * @param peerconnection The {@link PeerconnectionModel} to be formatted.
 * @returns The resulting {@link PeerconnectionOverview}.
 */
function formatPeerconnectionOverview(peerconnection) {
    return {
        devices: [
            formatDeviceReference(peerconnection.deviceA),
            formatDeviceReference(peerconnection.deviceB),
        ],
        url: (0, utils_1.peerconnectionUrlFromId)(peerconnection.uuid),
        status: peerconnection.status,
    };
}
exports.formatPeerconnectionOverview = formatPeerconnectionOverview;
/**
 * This function formats a {@link PeerconnectionModel} to a {@link Peerconnection}.
 * @param peerconnection The {@link PeerconnectionModel} to be formatted.
 * @returns The resulting {@link Peerconnection}.
 */
function formatPeerconnection(peerconnection) {
    return {
        devices: [
            formatConfiguredDeviceReference(peerconnection.deviceA),
            formatConfiguredDeviceReference(peerconnection.deviceB),
        ],
        url: (0, utils_1.peerconnectionUrlFromId)(peerconnection.uuid),
        status: peerconnection.status,
    };
}
exports.formatPeerconnection = formatPeerconnection;
//# sourceMappingURL=format.js.map