"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.writePeerconnection = exports.writeConfiguredDeviceReference = exports.writeServiceConfig = exports.writeInstantiableCloudDevice = exports.writeInstantiableBrowserDevice = exports.writeDeviceGroup = exports.writeConcreteDevice = exports.writeDeviceOverview = exports.writeAvailabilityRule = exports.writeDevice = void 0;
const data_source_1 = require("../../data_source");
const types_1 = require("../../generated/types");
const globals_1 = require("../../globals");
const model_1 = require("../../model");
const errors_1 = require("../../types/errors");
const availability_1 = require("../availability");
// Devices
/**
 * This function attempts to write the data of a device to a device model.
 * @param deviceModel The device model the data should be written to.
 * @param device The device providing the data to be written.
 * @throws {InvalidValueError} Thrown when the given device model and device do not
 * match any known device type or they have a different type from one another.
 */
function writeDevice(deviceModel, device) {
    if ((0, model_1.isConcreteDeviceModel)(deviceModel) && (0, types_1.isConcreteDevice)(device)) {
        writeConcreteDevice(deviceModel, device);
    }
    else if ((0, model_1.isDeviceGroupModel)(deviceModel) && (0, types_1.isDeviceGroup)(device)) {
        writeDeviceGroup(deviceModel, device);
    }
    else if ((0, model_1.isInstantiableBrowserDeviceModel)(deviceModel) &&
        (0, types_1.isInstantiableBrowserDevice)(device, "request")) {
        writeInstantiableBrowserDevice(deviceModel, device);
    }
    else if ((0, model_1.isInstantiableCloudDeviceModel)(deviceModel) &&
        (0, types_1.isInstantiableCloudDevice)(device)) {
        writeInstantiableCloudDevice(deviceModel, device);
    }
    else {
        throw new errors_1.InvalidValueError('The device model and device to be written do not match any known device type or they have a different type from one another', 500);
    }
}
exports.writeDevice = writeDevice;
/**
 * This function writes the data of an {@link AvailabilityRule} to an {@link AvailabilityRuleModel}.
 * @param availabilityRuleModel The {@link AvailabilityRuleModel} the data should be written to.
 * @param availabilityRule The {@link AvailabilityRule} providing the data to be written.
 */
function writeAvailabilityRule(availabilityRuleModel, availabilityRule) {
    availabilityRuleModel.available = availabilityRule.available ?? true;
    availabilityRuleModel.start = availabilityRule.start
        ? new Date(availabilityRule.start).getTime()
        : undefined;
    availabilityRuleModel.end = availabilityRule.end
        ? new Date(availabilityRule.end).getTime()
        : undefined;
    if (availabilityRule.repeat) {
        availabilityRuleModel.frequency = availabilityRule.repeat.frequency;
        availabilityRuleModel.until = availabilityRule.repeat.until
            ? new Date(availabilityRule.repeat.until).getTime()
            : undefined;
        availabilityRuleModel.count = availabilityRule.repeat.count;
    }
}
exports.writeAvailabilityRule = writeAvailabilityRule;
/**
 * This function writes the data of a {@link DeviceOverview} to a {@link DeviceOverviewModel}.
 * @param deviceOverviewModel The {@link DeviceOverviewModel} the data should be written to.
 * @param deviceOverview The {@link DeviceOverview} providing the data to be written.
 */
function writeDeviceOverview(deviceOverviewModel, deviceOverview) {
    if (deviceOverview.name)
        deviceOverviewModel.name = deviceOverview.name;
    if (deviceOverview.description)
        deviceOverviewModel.description = deviceOverview.description;
    if (deviceOverview.type)
        deviceOverviewModel.type = deviceOverview.type;
}
exports.writeDeviceOverview = writeDeviceOverview;
/**
 * This function writes the data of a {@link ConcreteDevice} to a {@link ConcreteDeviceModel}.
 * @param concreteDeviceModel The {@link ConcreteDeviceModel} the data should be written to.
 * @param concreteDevice The {@link ConcreteDevice} providing the data to be written.
 */
function writeConcreteDevice(concreteDeviceModel, concreteDevice) {
    writeDeviceOverview(concreteDeviceModel, concreteDevice);
    concreteDeviceModel.connected = false;
    concreteDeviceModel.token = undefined;
    concreteDeviceModel.experiment = concreteDevice.experiment;
    if (concreteDevice.services) {
        if (!concreteDeviceModel.services)
            concreteDeviceModel.services = [];
        const serviceRepository = data_source_1.AppDataSource.getRepository(model_1.ServiceModel);
        for (const service of concreteDevice.services) {
            const serviceModel = serviceRepository.create();
            serviceModel.description = JSON.stringify(service);
            concreteDeviceModel.services.push(serviceModel);
        }
    }
    if (concreteDevice.announcedAvailability) {
        if (!concreteDeviceModel.availabilityRules)
            concreteDeviceModel.availabilityRules = [];
        const availabilityRuleRepository = data_source_1.AppDataSource.getRepository(model_1.AvailabilityRuleModel);
        for (const availabilityRule of concreteDevice.announcedAvailability) {
            const availabilityRuleModel = availabilityRuleRepository.create();
            writeAvailabilityRule(availabilityRuleModel, availabilityRule);
            concreteDeviceModel.availabilityRules.push(availabilityRuleModel);
        }
    }
    if (!concreteDeviceModel.announcedAvailability)
        concreteDeviceModel.announcedAvailability = [];
    if (concreteDeviceModel.availabilityRules) {
        concreteDeviceModel.announcedAvailability = [];
        const start = Date.now();
        const end = start + globals_1.YEAR;
        concreteDeviceModel.announcedAvailability = (0, availability_1.applyAvailabilityRules)(concreteDeviceModel.announcedAvailability, concreteDeviceModel.availabilityRules, start, end);
    }
}
exports.writeConcreteDevice = writeConcreteDevice;
/**
 * This function writes the data of a {@link DeviceGroup} to a {@link DeviceGroupModel}.
 * @param deviceGroupModel The {@link DeviceGroupModel} the data should be written to.
 * @param deviceGroup The {@link DeviceGroup} providing the data to be written.
 */
function writeDeviceGroup(deviceGroupModel, deviceGroup) {
    writeDeviceOverview(deviceGroupModel, deviceGroup);
    if (deviceGroup.devices) {
        if (!deviceGroupModel.devices)
            deviceGroupModel.devices = [];
        const deviceReferenceRepository = data_source_1.AppDataSource.getRepository(model_1.DeviceReferenceModel);
        for (const deviceReference of deviceGroup.devices) {
            const deviceReferenceModel = deviceReferenceRepository.create();
            deviceReferenceModel.url = deviceReference.url ?? '';
            deviceGroupModel.devices.push(deviceReferenceModel);
        }
    }
}
exports.writeDeviceGroup = writeDeviceGroup;
/**
 * This function writes the data of a {@link instantiableBrowserDevice} to a {@link InstantiableBrowserDeviceModel}.
 * @param instantiableBrowserDeviceModel The {@link InstantiableBrowserDeviceModel} the data should be written to.
 * @param instantiableBrowserDevice The {@link instantiableBrowserDevice} providing the data to be written.
 */
function writeInstantiableBrowserDevice(instantiableBrowserDeviceModel, instantiableBrowserDevice) {
    writeDeviceOverview(instantiableBrowserDeviceModel, instantiableBrowserDevice);
    instantiableBrowserDeviceModel.codeUrl = instantiableBrowserDevice.code_url;
}
exports.writeInstantiableBrowserDevice = writeInstantiableBrowserDevice;
/**
 * This function writes the data of a {@link InstantiableCloudDevice} to a {@link InstantiableCloudDeviceModel}.
 * @param instantiableCloudDeviceModel The {@link InstantiableCloudDeviceModel} the data should be written to.
 * @param instantiableCloudDevice The {@link InstantiableCloudDevice} providing the data to be written.
 */
function writeInstantiableCloudDevice(instantiableCloudDeviceModel, instantiableCloudDevice) {
    writeDeviceOverview(instantiableCloudDeviceModel, instantiableCloudDevice);
    instantiableCloudDeviceModel.instantiateUrl = instantiableCloudDevice.instantiate_url;
}
exports.writeInstantiableCloudDevice = writeInstantiableCloudDevice;
// Peerconnections
/**
 * This function writes the data of a {@link ServiceConfig} to a {@link ServiceConfigModel}.
 * @param serviceConfigModel The {@link ServiceConfigModel} the data should be written to.
 * @param serviceConfig The {@link ServiceConfig} providing the data to be written.
 */
function writeServiceConfig(serviceConfigModel, serviceConfig) {
    serviceConfigModel.serviceType = serviceConfig.serviceType;
    serviceConfigModel.serviceId = serviceConfig.serviceId;
    serviceConfigModel.remoteServiceId = serviceConfig.remoteServiceId;
    serviceConfigModel.config = JSON.stringify(serviceConfig);
}
exports.writeServiceConfig = writeServiceConfig;
/**
 * This function writes the data of a {@link ConfiguredDeviceReference} to a {@link DeviceReferenceModel}.
 * @param configuredDeviceReferenceModel The {@link DeviceReferenceModel} the data should be written to.
 * @param configuredDeviceReference The {@link ConfiguredDeviceReference} providing the data to be written.
 */
async function writeConfiguredDeviceReference(configuredDeviceReferenceModel, configuredDeviceReference) {
    const serviceConfigRepository = data_source_1.AppDataSource.getRepository(model_1.ServiceConfigModel);
    if (configuredDeviceReference.url)
        configuredDeviceReferenceModel.url = configuredDeviceReference.url;
    if (configuredDeviceReference.config && configuredDeviceReference) {
        if (configuredDeviceReference.config.services) {
            configuredDeviceReferenceModel.config = [];
            for (const serviceConfig of configuredDeviceReference.config.services) {
                const serviceConfigModel = serviceConfigRepository.create();
                writeServiceConfig(serviceConfigModel, serviceConfig);
                configuredDeviceReferenceModel.config.push(serviceConfigModel);
            }
        }
    }
}
exports.writeConfiguredDeviceReference = writeConfiguredDeviceReference;
/**
 * This function writes the data of a {@link Peerconnection} to a {@link PeerconnectionModel}.
 * @param peerconnectionModel The {@link PeerconnectionModel} the data should be written to.
 * @param peerconnection The {@link Peerconnection} providing the data to be written.
 */
async function writePeerconnection(peerconnectionModel, peerconnection) {
    peerconnectionModel.status = 'waiting-for-devices';
    if (peerconnection.devices) {
        const deviceReferenceRepository = data_source_1.AppDataSource.getRepository(model_1.DeviceReferenceModel);
        const deviceReferenceA = deviceReferenceRepository.create();
        const deviceReferenceB = deviceReferenceRepository.create();
        await writeConfiguredDeviceReference(deviceReferenceA, peerconnection.devices[0]);
        await writeConfiguredDeviceReference(deviceReferenceB, peerconnection.devices[1]);
        peerconnectionModel.deviceA = deviceReferenceA;
        peerconnectionModel.deviceB = deviceReferenceB;
    }
}
exports.writePeerconnection = writePeerconnection;
//# sourceMappingURL=write.js.map