"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.deletePeerconnectionsByPeerconnectionId = exports.getPeerconnectionsByPeerconnectionId = exports.postPeerconnections = exports.getPeerconnections = exports.postDevicesByDeviceIdSignaling = exports.postDevicesByDeviceIdWebsocket = exports.postDevicesByDeviceIdAvailability = exports.patchDevicesByDeviceId = exports.deleteDevicesByDeviceId = exports.postDevicesByDeviceId = exports.getDevicesByDeviceId = exports.postDevices = exports.getDevices = exports.deviceHandling = exports.connectedDevices = void 0;
const data_source_1 = require("./data_source");
const model_1 = require("./model");
const crypto_1 = require("crypto");
const globals_1 = require("./globals");
const errors_1 = require("./types/errors");
const utils_1 = require("./methods/utils");
const types_1 = require("./generated/types");
const availability_1 = require("./methods/availability");
const callbacks_1 = require("./methods/callbacks");
const format_1 = require("./methods/database/format");
const messageHandling_1 = require("./methods/messageHandling");
const write_1 = require("./methods/database/write");
const create_1 = require("./methods/database/create");
const delete_1 = require("./methods/database/delete");
const save_1 = require("./methods/database/save");
const find_1 = require("./methods/database/find");
const signaling_1 = require("./methods/signaling");
exports.connectedDevices = new Map();
/**
 * This function adds the /devices/ws endpoint, including its functionality, to an express application.
 * @param app The express application to add the /devices/ws endpoint to.
 */
function deviceHandling(app) {
    // TODO: close Peerconnections that have device as participant when websocket connection is closed?
    app.ws('/devices/websocket', (ws) => {
        // authenticate and start heartbeat
        ws.once('message', async (data) => {
            // device authentication and connection
            const deviceRepository = data_source_1.AppDataSource.getRepository(model_1.ConcreteDeviceModel);
            const message = JSON.parse(data.toString('utf8'));
            if (!((0, types_1.isMessage)(message) && (0, types_1.isAuthenticationMessage)(message))) {
                ws.close(1002, 'Received message is not an authentication message');
                return;
            }
            if (!message.token) {
                ws.close(1002, 'Authentication message does not contain a valid websocket token');
            }
            const device = await deviceRepository.findOne({ where: { token: message.token } });
            if (!device) {
                ws.close(1002, 'No device found with matching websocket token');
                return;
            }
            if (device.token != message.token) {
                ws.close(1002, 'Provided token does not match the token of the device');
                return;
            }
            device.connected = true;
            exports.connectedDevices.set(device.uuid, ws);
            await deviceRepository.save(device);
            await (0, callbacks_1.sendChangedCallback)(device);
            console.log(`device ${device.uuid} connected`);
            ws.send(JSON.stringify({
                messageType: 'authenticate',
                authenticated: true,
            }));
            // heartbeat implementation
            let isAlive = true;
            ws.on('pong', () => {
                isAlive = true;
            });
            const interval = setInterval(async function ping() {
                if (isAlive === false) {
                    device.connected = false;
                    await deviceRepository.save(device);
                    await (0, callbacks_1.sendChangedCallback)(device);
                    exports.connectedDevices.delete(device.uuid);
                    clearInterval(interval);
                    return ws.terminate();
                }
                isAlive = false;
                ws.ping();
            }, 30000);
            // close handler: stop heartbeat and disconnect device
            ws.on('close', async (code, reason) => {
                clearInterval(interval);
                exports.connectedDevices.delete(device.uuid);
                if (code === 1002) {
                    console.error(`WebSocketConnctionError "${reason}"`);
                }
            });
            // message handler: handle incoming messages from devices
            ws.on('message', async (data) => {
                const message = JSON.parse(data.toString('utf8'));
                if (!(0, types_1.isMessage)(message)) {
                    ws.close(1002, 'Malformed Message');
                    return;
                }
                (0, messageHandling_1.handleDeviceMessage)(device, message);
            });
        });
    });
}
exports.deviceHandling = deviceHandling;
/**
 * This function implements the functionality for handling GET requests on /devices endpoint.
 * @param _user The user submitting the request.
 */
const getDevices = async (_user) => {
    console.log(`getDevices called`);
    const deviceRepository = data_source_1.AppDataSource.getRepository(model_1.DeviceOverviewModel);
    const devices = await deviceRepository.find();
    console.log(`getDevices succeeded`);
    return {
        status: 200,
        body: devices.map(format_1.formatDeviceOverview),
    };
};
exports.getDevices = getDevices;
/**
 * This function implements the functionality for handling POST requests on /devices endpoint.
 * @param parameters The parameters of the request.
 * @param body The body of the request.
 * @param user The user submitting the request.
 */
const postDevices = async (parameters, body, user) => {
    console.log(`postDevices called`);
    const device = (0, create_1.createDeviceModelFromDevice)(body);
    device.owner = user.JWT?.url;
    await (0, save_1.saveDeviceModel)(device);
    if (parameters.changedUrl) {
        console.log(`registering changed-callback for device ${device.uuid} to ${parameters.changedUrl}`);
        const changedCallbackURLs = callbacks_1.changedCallbacks.get(device.uuid) ?? [];
        changedCallbackURLs.push(parameters.changedUrl);
        callbacks_1.changedCallbacks.set(device.uuid, changedCallbackURLs);
    }
    console.log(`postDevices succeeded`);
    return {
        status: 201,
        body: await (0, format_1.formatDevice)(device),
    };
};
exports.postDevices = postDevices;
/**
 * This function implements the functionality for handling GET requests on /devices/{device_id} endpoint.
 * @param parameters The parameters of the request.
 * @param _user The user submitting the request.
 * @throws {MissingEntityError} Thrown if device is not found in the database.
 */
const getDevicesByDeviceId = async (parameters, _user) => {
    console.log(`getDevicesByDeviceId called`);
    const device = await (0, find_1.findDeviceModelByUUID)(parameters.device_id);
    if (!device)
        throw new errors_1.MissingEntityError(`Could not find device ${parameters.device_id}`, 404);
    console.log(`getDevicesByDeviceId succeeded`);
    return {
        status: 200,
        body: await (0, format_1.formatDevice)(device),
    };
};
exports.getDevicesByDeviceId = getDevicesByDeviceId;
/**
 * This function implements the functionality for handling POST requests on /devices/{device_id} endpoint.
 * @param parameters The parameters of the request.
 * @param user The user submitting the request.
 * @throws {MissingEntityError} Thrown if device is not found in the database.
 * @throws {ForbiddenOperationError} Thrown if device is not instantiable.
 */
const postDevicesByDeviceId = async (parameters, user) => {
    console.log(`postDevicesByDeviceId called`);
    const instantiableDevice = await (0, find_1.findDeviceModelByUUID)(parameters.device_id);
    if (!instantiableDevice)
        throw new errors_1.MissingEntityError(`Could not find device ${parameters.device_id}`, 404);
    if (!(0, model_1.isInstantiableDeviceModel)(instantiableDevice))
        throw new errors_1.ForbiddenOperationError(`Cannot create new instance of device ${(0, utils_1.deviceUrlFromId)(instantiableDevice.uuid)} since it has type "${instantiableDevice.type}"`, 400);
    const concreteDevice = {
        ...instantiableDevice,
        type: 'device',
        announcedAvailability: [{ available: true }],
        services: [],
    };
    const concreteDeviceModel = (0, create_1.createDeviceModelFromDevice)(concreteDevice);
    concreteDeviceModel.owner = user.JWT?.url;
    await (0, save_1.saveDeviceModel)(concreteDeviceModel);
    if (parameters.changedUrl) {
        console.log(`registering changed-callback for device ${concreteDevice.uuid} to ${parameters.changedUrl}`);
        const changedCallbackURLs = callbacks_1.changedCallbacks.get(concreteDeviceModel.uuid) ?? [];
        changedCallbackURLs.push(parameters.changedUrl);
        callbacks_1.changedCallbacks.set(concreteDeviceModel.uuid, changedCallbackURLs);
    }
    const instance = await (0, format_1.formatDevice)(concreteDeviceModel);
    if (!instance.url)
        throw new errors_1.MissingPropertyError('Created instance of device does not have an url', 500);
    const deviceToken = await globals_1.apiClient.createDeviceAuthenticationToken(instance.url); // TODO: error handling
    if (!instantiableDevice.instances)
        instantiableDevice.instances = [];
    instantiableDevice.instances.push(concreteDeviceModel);
    await (0, save_1.saveDeviceModel)(instantiableDevice);
    console.log(`postDevicesByDeviceId succeeded`);
    return {
        status: 201,
        body: {
            instance,
            deviceToken,
        },
    };
};
exports.postDevicesByDeviceId = postDevicesByDeviceId;
/**
 * This function implements the functionality for handling DELETE requests on /devices/{device_id} endpoint.
 * @param parameters The parameters of the request.
 * @param _user The user submitting the request.
 * @throws {MissingEntityError} Thrown if device is not found in the database.
 */
const deleteDevicesByDeviceId = async (parameters, _user) => {
    console.log(`deleteDevicesByDeviceId called`);
    const device = await (0, find_1.findDeviceModelByUUID)(parameters.device_id);
    if (!device)
        throw new errors_1.MissingEntityError(`Could not find device ${parameters.device_id}`, 404);
    await (0, delete_1.deleteDeviceModel)(device);
    console.log(`deleteDevicesByDeviceId succeeded`);
    return {
        status: 204,
    };
};
exports.deleteDevicesByDeviceId = deleteDevicesByDeviceId;
/**
 * This function implements the functionality for handling PATCH requests on /devices/{device_id} endpoint.
 * @param parameters The parameters of the request.
 * @param body The body of the request.
 * @param _user The user submitting the request.
 * @throws {MissingEntityError} Thrown if device is not found in the database.
 * @throws {InvalidChangeError} Thrown if client tries to update the type of the device.
 */
const patchDevicesByDeviceId = async (parameters, body, _user) => {
    console.log(`patchDevicesByDeviceId called`);
    const device = await (0, find_1.findDeviceModelByUUID)(parameters.device_id);
    if (!device)
        throw new errors_1.MissingEntityError(`Could not find device ${parameters.device_id}`, 404);
    if (parameters.changedUrl) {
        console.log(`registering changed-callback for device ${device.uuid} to ${parameters.changedUrl}`);
        const changedCallbackURLs = callbacks_1.changedCallbacks.get(device.uuid) ?? [];
        changedCallbackURLs.push(parameters.changedUrl);
        callbacks_1.changedCallbacks.set(device.uuid, changedCallbackURLs);
    }
    if (!body || Object.keys(body).length === 0) {
        console.log(`patchDevicesByDeviceId succeeded: no changes applied due to empty body`);
        return {
            status: 200,
            body: await (0, format_1.formatDevice)(device),
        };
    }
    if (!device.type) {
        throw new errors_1.MissingPropertyError(`Device model is missing a type`);
    }
    (0, write_1.writeDevice)(device, body);
    await (0, callbacks_1.sendChangedCallback)(device);
    console.log(`patchDevicesByDeviceId succeeded`);
    return {
        status: 200,
        body: await (0, format_1.formatDevice)(device),
    };
};
exports.patchDevicesByDeviceId = patchDevicesByDeviceId;
/**
 * This function implements the functionality for handling POST requests on /devices/{device_id}/availability endpoint.
 * @param parameters The parameters of the request.
 * @param body The body of the request.
 * @param _user The user submitting the request.
 * @throws {MissingEntityError} Thrown if device is not found in the database.
 */
const postDevicesByDeviceIdAvailability = async (parameters, body, _user) => {
    console.log(`postDevicesByDeviceIdAvailability called`);
    const deviceRepository = data_source_1.AppDataSource.getRepository(model_1.ConcreteDeviceModel);
    const device = await deviceRepository.findOneBy({ uuid: parameters.device_id });
    if (!device)
        throw new errors_1.MissingEntityError(`Could not find device ${parameters.device_id}`, 404);
    const availabilityRuleRepository = data_source_1.AppDataSource.getRepository(model_1.AvailabilityRuleModel);
    if (!device.availabilityRules)
        device.availabilityRules = [];
    if (body) {
        // insert new availability rules
        for (const availabilityRule of body) {
            const availabilityRuleModel = availabilityRuleRepository.create();
            (0, write_1.writeAvailabilityRule)(availabilityRuleModel, availabilityRule);
            device.availabilityRules.push(availabilityRuleModel);
        }
    }
    device.announcedAvailability = [];
    const start = Date.now();
    const end = start + globals_1.YEAR;
    device.announcedAvailability = (0, availability_1.applyAvailabilityRules)(device.announcedAvailability, device.availabilityRules, start, end);
    await deviceRepository.save(device);
    await (0, callbacks_1.sendChangedCallback)(device);
    console.log(`postDevicesByDeviceIdAvailability succeeded`);
    return {
        status: 200,
        body: device.announcedAvailability.map(format_1.formatTimeSlot),
    };
};
exports.postDevicesByDeviceIdAvailability = postDevicesByDeviceIdAvailability;
/**
 * This function implements the functionality for handling POST requests on /devices/{device_id}/token endpoint.
 * @param parameters The parameters of the request.
 * @param _user The user submitting the request.
 * @throws {MissingEntityError} Thrown if device is not found in the database.
 */
const postDevicesByDeviceIdWebsocket = async (parameters, _user) => {
    console.log(`postDevicesByDeviceIdToken called`);
    const deviceRepository = data_source_1.AppDataSource.getRepository(model_1.ConcreteDeviceModel);
    const device = await deviceRepository.findOneBy({ uuid: parameters.device_id });
    if (!device)
        throw new errors_1.MissingEntityError(`Could not find device ${parameters.device_id}`, 404);
    device.token = (0, crypto_1.randomUUID)();
    await deviceRepository.save(device);
    setTimeout(async () => {
        device.token = undefined;
        await deviceRepository.save(device);
    }, 300000);
    console.log(`postDevicesByDeviceIdToken succeeded`);
    return {
        status: 200,
        body: device.token,
    };
};
exports.postDevicesByDeviceIdWebsocket = postDevicesByDeviceIdWebsocket;
/**
 * This function implements the functionality for handling POST requests on /devices/{device_id}/signaling endpoint.
 * @param parameters The parameters of the request.
 * @param body The body of the request.
 * @param _user The user submitting the request.
 * @throws {MissingEntityError} Thrown if device is not found in the database or if websocket for device is not found.
 * @throws {InvalidValueError} Thrown if type of device is not "device" or if device is not part of the peerconnection.
 */
const postDevicesByDeviceIdSignaling = async (parameters, body, _user) => {
    console.log(`postDevicesByDeviceIdSignaling called`);
    // Get device
    const device = await (0, find_1.findDeviceModelByUUID)(parameters.device_id);
    if (!device)
        throw new errors_1.MissingEntityError(`Could not find device ${parameters.device_id}`, 404);
    // Make sure device is a concrete device
    if (device.type !== 'device')
        throw new errors_1.ForbiddenOperationError(`Cannot send signaling message to device with type ${device.type}`, 400);
    // Retrieve peerconnection and make sure the device is taking part in it
    const peerconnection = await globals_1.apiClient.getPeerconnection(parameters.peerconnection_url);
    if (!peerconnection.devices)
        throw new errors_1.MissingPropertyError(`Peerconnection does not have any devices`, 404);
    const deviceA = peerconnection.devices[0];
    const deviceB = peerconnection.devices[1];
    if (!(deviceA.url === (0, utils_1.deviceUrlFromId)(device.uuid)) &&
        !(deviceB.url === (0, utils_1.deviceUrlFromId)(device.uuid))) {
        throw new errors_1.UnrelatedPeerconnectionError(`Device is not part of the peerconnection`, 400);
    }
    const ws = exports.connectedDevices.get(parameters.device_id);
    if (!ws)
        throw new errors_1.MissingEntityError(`Could not find websocket connection for device ${parameters.device_id}`, 404);
    ws.send(JSON.stringify(body));
    console.log(`postDevicesByDeviceIdSignaling succeeded`);
    return {
        status: 200,
    };
};
exports.postDevicesByDeviceIdSignaling = postDevicesByDeviceIdSignaling;
/**
 * This function implements the functionality for handling GET requests on /peerconnections endpoint.
 * @param _user The user submitting the request.
 */
const getPeerconnections = async (_user) => {
    console.log(`getPeerconnections called`);
    const peerconnectionRepository = data_source_1.AppDataSource.getRepository(model_1.PeerconnectionModel);
    const peerconnections = await peerconnectionRepository.find({
        relations: {
            deviceA: true,
            deviceB: true,
        },
    });
    console.log(`getPeerconnections succeeded`);
    return {
        status: 200,
        body: peerconnections.map(format_1.formatPeerconnectionOverview),
    };
};
exports.getPeerconnections = getPeerconnections;
/**
 * This function implements the functionality for handling POST requests on /peerconnections endpoint.
 * @param parameters The parameters of the request.
 * @param body The body of the request.
 * @param _user The user submitting the request.
 */
const postPeerconnections = async (parameters, body, _user) => {
    console.log(`postPeerconnections called`);
    const peerconnectionRepository = data_source_1.AppDataSource.getRepository(model_1.PeerconnectionModel);
    const peerconnection = peerconnectionRepository.create();
    await (0, write_1.writePeerconnection)(peerconnection, body);
    if (!peerconnection.deviceA.url || !peerconnection.deviceB.url) {
        throw new errors_1.MissingEntityError(`One of the participating devices has no url`, 404);
    }
    await peerconnectionRepository.save(peerconnection);
    const deviceA = await globals_1.apiClient.getDevice(peerconnection.deviceA.url);
    const deviceB = await globals_1.apiClient.getDevice(peerconnection.deviceB.url);
    if (deviceA.type !== 'device' || deviceB.type !== 'device') {
        throw new errors_1.InvalidValueError(`Cannot establish a peerconnection between devices of type "${deviceA.type}" and "${deviceB.type}"`, 400);
    }
    if (!deviceA.url || !deviceB.url) {
        throw new errors_1.MissingPropertyError(`One of the resolved devices does not have an url`, 500); // NOTE: error code
    }
    if (deviceA.connected && deviceB.connected) {
        // peerconnection can be started directly
        await (0, signaling_1.startSignaling)(peerconnection);
    }
    else {
        // need to wait for devices to connect
        // register changed callbacks for devices to get notified when they connect
        const n_deviceA = await globals_1.apiClient.updateDevice(deviceA.url, {}, { changedUrl: callbacks_1.callbackUrl });
        const n_deviceB = await globals_1.apiClient.updateDevice(deviceB.url, {}, { changedUrl: callbacks_1.callbackUrl });
        // check that devices still have the correct type
        if (n_deviceA.type !== 'device' || n_deviceB.type !== 'device') {
            throw new errors_1.InvalidValueError(`The type of device ${deviceA.type !== 'device' ? deviceA.url : deviceB.url} is not "device" anymore`, 400);
        }
        // set timeout for checking if devices are connected
        globals_1.timeoutMap.set(peerconnection.uuid, setTimeout(async () => {
            console.log('devices did not connect');
            peerconnection.status = 'failed';
            await peerconnectionRepository.save(peerconnection);
            await (0, callbacks_1.sendStatusChangedCallback)(peerconnection);
        }, 30000));
    }
    if (parameters.closedUrl) {
        console.log(`postPeerconnections: registering closed-callback for ${parameters.closedUrl}`);
        const closedCallbackURLs = callbacks_1.closedCallbacks.get(peerconnection.uuid) ?? [];
        closedCallbackURLs.push(parameters.closedUrl);
        callbacks_1.closedCallbacks.set(peerconnection.uuid, closedCallbackURLs);
    }
    if (parameters.statusChangedUrl) {
        console.log(`postPeerconnections: registering status-changed-callback for ${parameters.statusChangedUrl}`);
        const statusChangedCallbackURLs = callbacks_1.statusChangedCallbacks.get(peerconnection.uuid) ?? [];
        statusChangedCallbackURLs.push(parameters.statusChangedUrl);
        callbacks_1.statusChangedCallbacks.set(peerconnection.uuid, statusChangedCallbackURLs);
    }
    console.log(`postPeerconnections succeeded`);
    return {
        status: peerconnection.status === 'connected' ? 201 : 202,
        body: (0, format_1.formatPeerconnectionOverview)(peerconnection),
    };
};
exports.postPeerconnections = postPeerconnections;
/**
 * This function implements the functionality for handling GET requests on /peerconnections/{peerconnection_id} endpoint.
 * @param parameters The parameters of the request.
 * @param _user The user submitting the request.
 */
const getPeerconnectionsByPeerconnectionId = async (parameters, _user) => {
    console.log(`getPeerconnectionsByPeerconnectionId called`);
    const peerconnectionRepository = data_source_1.AppDataSource.getRepository(model_1.PeerconnectionModel);
    const peerconnection = await peerconnectionRepository.findOne({
        where: {
            uuid: parameters.peerconnection_id,
        },
        relations: {
            deviceA: true,
            deviceB: true,
        },
    });
    if (!peerconnection)
        throw new errors_1.MissingEntityError(`Could not find peerconnection ${parameters.peerconnection_id}`, 404);
    console.log(`getPeerconnectionsByPeerconnectionId succeeded`);
    return {
        status: 200,
        body: (0, format_1.formatPeerconnectionOverview)(peerconnection),
    };
};
exports.getPeerconnectionsByPeerconnectionId = getPeerconnectionsByPeerconnectionId;
/**
 * This function implements the functionality for handling DELETE requests on /peerconnection/{peerconnection_id} endpoint.
 * @param parameters The parameters of the request.
 * @param _user The user submitting the request.
 */
const deletePeerconnectionsByPeerconnectionId = async (parameters, _user) => {
    console.log(`deletePeerconnectionsByPeerconnectionId called`);
    const peerconnectionRepository = data_source_1.AppDataSource.getRepository(model_1.PeerconnectionModel);
    const peerconnection = await peerconnectionRepository.findOneOrFail({
        where: {
            uuid: parameters.peerconnection_id,
        },
        relations: {
            deviceA: true,
            deviceB: true,
        },
    });
    const result = await peerconnectionRepository.softDelete({
        uuid: parameters.peerconnection_id,
    });
    if (!result.affected) {
        throw new errors_1.MissingEntityError(`Could not find peerconnection ${parameters.peerconnection_id}`, 404);
    }
    if (result.affected > 1) {
        throw new errors_1.InconsistentDatabaseError('Deleted Multiple Peerconnections with same uuid', 500);
    }
    const closePeerconnectionMessage = {
        messageType: 'command',
        command: 'closePeerconnection',
        connectionUrl: (0, utils_1.peerconnectionUrlFromId)(peerconnection.uuid),
    };
    await globals_1.apiClient.sendSignalingMessage(peerconnection.deviceA.url, closePeerconnectionMessage, (0, utils_1.peerconnectionUrlFromId)(peerconnection.uuid));
    await globals_1.apiClient.sendSignalingMessage(peerconnection.deviceB.url, closePeerconnectionMessage, (0, utils_1.peerconnectionUrlFromId)(peerconnection.uuid));
    await (0, callbacks_1.sendClosedCallback)(peerconnection);
    console.log(`deletePeerconnectionsByPeerconnectionId succeeded`);
    return {
        status: 204,
    };
};
exports.deletePeerconnectionsByPeerconnectionId = deletePeerconnectionsByPeerconnectionId;
//# sourceMappingURL=operations.js.map